package fmon.world

import java.io._

import fmon.draw.tile._
import fmon.util._

class GameMap(val width: Int, val height: Int, val tileset: Tileset) {
  
  var tiles: IndexedSeq[AutoTile] = for (y <- 0 until height; x <- 0 until width) yield tileset.groundTiles.head
  
  def saveTo(file: File): Unit = {
    val ostream = new ObjectOutputStream(new BufferedOutputStream(new FileOutputStream(file)))
    ostream.writeInt(width)
    ostream.writeInt(height)
    ostream.writeObject(tileset.token.a2)// Tileset
    val tileIndices = tileset.groundTiles.zipWithIndex.toMap
    tiles.foreach(t => ostream.writeInt(tileIndices(t)))
    //ostream.writeObject(tiles.map(tileIndices(_)).mkString(" "))
    ostream.close()
  }
  
  def pt2index(x: Int, y: Int) = y * width + x
  def index2x(i: Int) = i % width
  def index2y(i: Int) = i / width
}

object GameMap {
  def loadFrom(file: File): GameMap = {
    //val istream = new BufferedReader(new FileReader(file))
    val istream = new ObjectInputStream(new BufferedInputStream(new FileInputStream(file)))
    val width = istream.readInt()
    val height = istream.readInt()
    val tileset = new Tileset(new TilesetToken("Tileset", null, istream.readObject().toString, null, null, null, null, null, IndexedSeq()))
    val indices = for (y <- 0 until height; x <- 0 until width) yield istream.readInt()
    val map = new GameMap(width, height, tileset)
    val tiles = indices.map(i => tileset.groundTiles(i))
    map.tiles = tiles
    map
  }
}